% NLMAIN    Ng and Li's Test-Problem Generator Main Program (version 2.0)
%             (for internal use only)



function NL = nglimain(NL)


% Read random numbers form file
%
AllRand = read_rand(NL);


% In theory, a(i) > 0 
% In practice, we generate a(i) in [aLB, aUB] with exactly 16 digits
%
NL.a  = NL.aLB + (NL.aUB-NL.aLB) .* AllRand(1:NL.n);
Power = floor(log10(abs(NL.a)));
NL.a  = round(NL.a .* 10.^(15-Power)) ./ 10.^(15-Power);


% In theory, p(i) real
% In practice, we generate p(i) in [pLB, pUB] with exactly 16 digits
%
NL.p  = NL.pLB + (NL.pUB-NL.pLB) .* AllRand(NL.n+1:2*NL.n);
Power = floor(log10(abs(NL.p)));
NL.p  = round(NL.p .* 10.^(15-Power)) ./ 10.^(15-Power);


% In theory, q(i) such that q(i) < p(i)^2
% In practice, we generate q(i) in [qLB, qUB] with exactly 16 digits
% where qUB < 0
%
NL.q  = NL.qLB + (NL.qUB-NL.qLB) .* AllRand(2*NL.n+1:3*NL.n);
Power = floor(log10(abs(NL.q)));
NL.q  = round(NL.q .* 10.^(15-Power)) ./ 10.^(15-Power);


% Calculate r(i) = sqrt( p(i)^2-q(i) )
%
NL.r = sqrt(NL.p.^2 - NL.q);


% For difficult subproblems, we generate alpha(i) such that
%   alpha(i) in [-p(i)+(-2+0.5*LEN)*r(i), -p(i)+(-2+0.975*LEN)*r(i)]  OR
%   alpha(i) in [-p(i)+(sqrt(3)+0.025*LEN)*r(i), -p(i)+(sqrt(3)+0.5*LEN)*r(i)]
% For easy subproblems, we generate alpha(i) such that
%   alpha(i) in [-p(i)+(-2+0.025*LEN)*r(i), -p(i)+(-2+0.5*LEN)*r(i)]  OR
%   alpha(i) in [-p(i)+(sqrt(3)+0.5*LEN)*r(i), -p(i)+(sqrt(3)+0.975*LEN)*r(i)]
%
LEN = 2-sqrt(3);
NL.alpha  = zeros(NL.n,1);	% Initialize alpha
for I = 1:NL.nDiff
  RAND = AllRand(3*NL.n+I);	% rand in [0,1]
  if RAND < 0.5
    RAND = 2*RAND;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (-2+(1+NL.alphaPrime*RAND)*LEN/2) * NL.r(I);
  else
    RAND = 2*RAND-1;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (sqrt(3)+(1-NL.alphaPrime+NL.alphaPrime*RAND)*LEN/2) * NL.r(I);
  end
end
for I = NL.nDiff+1:NL.n
  RAND = AllRand(3*NL.n+I);	% rand in [0,1]
  if RAND < 0.5
    RAND = 2*RAND;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (-2+(1-NL.alphaPrime+NL.alphaPrime*RAND)*LEN/2) * NL.r(I);
  else
    RAND = 2*RAND-1;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (sqrt(3)+(1+NL.alphaPrime*RAND)*LEN/2) * NL.r(I);
  end
end


% Calculate s(i) with exactly 16 digits
%
NL.s  = -4*NL.alpha .* (NL.alpha.^2 + 3*NL.p.*NL.alpha + 3*NL.q);
Power = floor(log10(abs(NL.s)));
NL.s  = round(NL.s .* 10.^(15-Power)) ./ 10.^(15-Power);


% Calculate fStar
%
NL.fStar = sum(NL.a .* (NL.alpha.^4 + 4 * NL.p .* NL.alpha.^3 + 6 * NL.q .* NL.alpha.^2 + NL.s .* NL.alpha));


% Calculate eigenvalues of the Hessian matrix of f at alpha
%
NL.ddfEig = 12 * NL.a .* (NL.alpha.^2 + 2 * NL.p .* NL.alpha + NL.q);


% Calculate Delta, beta, gamma
%
NL.Delta  = 3*(2*NL.r + NL.p + NL.alpha).*(2*NL.r - NL.p - NL.alpha);
NL.beta   = (-(3*NL.p + NL.alpha) - sqrt(NL.Delta))/2;
NL.gamma  = (-(3*NL.p + NL.alpha) + sqrt(NL.Delta))/2;


% Check if alpha is global minimum or not
%
NL.fialpha = NL.alpha.^4 + 4*NL.p.*NL.alpha.^3 + 6*NL.q.*NL.alpha.^2 + NL.s.*NL.alpha;
NL.fibeta  = NL.beta.^4  + 4*NL.p.*NL.beta.^3  + 6*NL.q.*NL.beta.^2  + NL.s.*NL.beta;
NL.figamma = NL.gamma.^4 + 4*NL.p.*NL.gamma.^3 + 6*NL.q.*NL.gamma.^2 + NL.s.*NL.gamma;

NL.fiMax = max([NL.fibeta, NL.figamma], [], 2);
NL.fMax  = NL.a.' * NL.fiMax;

if all(NL.fialpha < NL.fibeta) && all(NL.fialpha < NL.figamma)
  NL.global = true;
else
  NL.global = false;
end




% START ALGORITHM 2




% In theory, d(i) > 0
% In practice, we generate d(i) in [dLB, dUB] with exactly 16 digits
%
NL.d  = NL.dLB + (NL.dUB-NL.dLB) .* AllRand(4*NL.n+1:5*NL.n);
Power = floor(log10(abs(NL.d)));
NL.d  = round(NL.d .* 10.^(15-Power)) ./ 10.^(15-Power);


% invD = inv(D) = inv(diag(d))
%
NL.invD = diag(1 ./ NL.d);


% In theory, v(i)>=0 and v'*v = sum(v(i)^2) = 1
% In practice, we first generate v(i) in [0,1]
% We then set v(i) := v(i) / sqrt(sum(v(i)^2)), then sum(v(i)^2) = 1
%
NL.v = AllRand(5*NL.n+1:6*NL.n);
NL.v = NL.v / sqrt(sum(NL.v.^2));
Power = floor(log10(abs(NL.v)));
NL.v  = round(NL.v .* 10.^(15-Power)) ./ 10.^(15-Power);


% Calculate H = I - 2* v * v'
%
NL.H = eye(NL.n) - 2 * (NL.v * NL.v');


% Calculate yStar = H * inv(D) * alpha
%
NL.yStar = NL.H * NL.invD * NL.alpha;


% Calculate eigenvalues and condition no. of the Hessian matrix of g at yStar
%
NL.ddgEig    = NL.d.^2 .* NL.ddfEig;
NL.ddgEigMin = min(NL.ddgEig);
NL.ddgCond   = max(NL.ddgEig)/min(NL.ddgEig);




% START ALGORITHM 3




% In theory, deltaL(i) > 0 and deltaR(i) > 0
% In practice, we generate deltaL and deltaR in [deltaLB, deltaUB]
%
NL.deltaL = NL.deltaLB + (NL.deltaUB-NL.deltaLB) .* AllRand(6*NL.n+1:7*NL.n);
NL.deltaR = NL.deltaLB + (NL.deltaUB-NL.deltaLB) .* AllRand(7*NL.n+1:8*NL.n);


% Calculate xL(i), xR(i), RL(i) and RR(i)
%
NL.xL = zeros(NL.n,1);
NL.xR = zeros(NL.n,1);
NL.RL = zeros(NL.n,1);
NL.RR = zeros(NL.n,1);
for I=1:NL.n
  if AllRand(3*NL.n+I) < 0.5
    NL.xL(I) = NL.alpha(I);
    NL.xR(I) = NL.gamma(I);
    NL.RL(I) = (-3*NL.p(I)-3*NL.alpha(I)-sqrt(NL.Delta(I)))/2;
    NL.RR(I) = sqrt(NL.Delta(I));
  else
    NL.xL(I) = NL.beta(I);
    NL.xR(I) = NL.alpha(I);
    NL.RL(I) = sqrt(NL.Delta(I));
    NL.RR(I) = (3*NL.p(I)+3*NL.alpha(I)-sqrt(NL.Delta(I)))/2;
  end
end


% Calculate xLB(i) and xUB(i)
%
NL.xLB = NL.xL - NL.deltaL .* NL.RL;
NL.xUB = NL.xR + NL.deltaR .* NL.RR;


% Calculate yLB & yUB with exactly 16 digits
%
NL.yLB      = zeros(NL.n,1);
NL.yUB      = zeros(NL.n,1);
v_xLB_d     = NL.v .* NL.xLB ./ NL.d;
v_xUB_d     = NL.v .* NL.xUB ./ NL.d;
sum_v_xLB_d = sum(v_xLB_d);
sum_v_xUB_d = sum(v_xUB_d);
for I = 1:NL.n
  if NL.v(I)^2 >= 0.5
    NL.yLB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xUB(I) - 2*NL.v(I)*(sum_v_xUB_d - NL.v(I)*NL.xUB(I)/NL.d(I));
    NL.yUB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xLB(I) - 2*NL.v(I)*(sum_v_xLB_d - NL.v(I)*NL.xLB(I)/NL.d(I));
  else
    NL.yLB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xLB(I) - 2*NL.v(I)*(sum_v_xUB_d - NL.v(I)*NL.xUB(I)/NL.d(I));
    NL.yUB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xUB(I) - 2*NL.v(I)*(sum_v_xLB_d - NL.v(I)*NL.xLB(I)/NL.d(I));
  end
end


% Calculate fUB(i)
%
NL.fixLB = NL.xLB.^4 + 4*NL.p.*NL.xLB.^3 + 6*NL.q.*NL.xLB.^2 + NL.s.*NL.xLB;
NL.fixUB = NL.xUB.^4 + 4*NL.p.*NL.xUB.^3 + 6*NL.q.*NL.xUB.^2 + NL.s.*NL.xUB;
NL.fiUB  = max([NL.fibeta, NL.figamma, NL.fixLB, NL.fixUB], [], 2);
NL.fUB   = NL.a.' * NL.fiUB;


% Calculate xLBLB(i) and xUBUB(i)
%
NL.xLBLB  = zeros(NL.n,1);
NL.xUBUB  = zeros(NL.n,1);
v_yLB     = NL.v .* NL.yLB;
v_yUB     = NL.v .* NL.yUB;
sum_v_yLB = sum(v_yLB);
sum_v_yUB = sum(v_yUB);
for I=1:NL.n
  if NL.v(I)^2 >= 0.5
    NL.xLBLB(I) = NL.d(I) * ((1-2*NL.v(I)^2)*NL.yUB(I) - 2*NL.v(I)*(sum_v_yUB - NL.v(I)*NL.yUB(I)));
    NL.xUBUB(I) = NL.d(I) * ((1-2*NL.v(I)^2)*NL.yLB(I) - 2*NL.v(I)*(sum_v_yLB - NL.v(I)*NL.yLB(I)));
  else
    NL.xLBLB(I) = NL.d(I) * ((1-2*NL.v(I)^2)*NL.yLB(I) - 2*NL.v(I)*(sum_v_yUB - NL.v(I)*NL.yUB(I)));
    NL.xUBUB(I) = NL.d(I) * ((1-2*NL.v(I)^2)*NL.yUB(I) - 2*NL.v(I)*(sum_v_yLB - NL.v(I)*NL.yLB(I)));
  end
end


% Calculate fUBUB(i)
%
NL.fixLBLB = NL.xLBLB.^4 + 4*NL.p.*NL.xLBLB.^3 + 6*NL.q.*NL.xLBLB.^2 + NL.s.*NL.xLBLB;
NL.fixUBUB = NL.xUBUB.^4 + 4*NL.p.*NL.xUBUB.^3 + 6*NL.q.*NL.xUBUB.^2 + NL.s.*NL.xUBUB;
NL.fiUBUB  = max([NL.fibeta, NL.figamma, NL.fixLBLB, NL.fixUBUB], [], 2);
NL.fUBUB   = NL.a.' * NL.fiUBUB;




% Write data to files
%
write_info(NL);
write_matlab(NL);
write_gams(NL);





function AllRand = read_rand(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.rn'], 'r');
if fid > 0
  AllRand = fscanf(fid, '%e', [1 inf])';
  fclose(fid);
else
  fprintf('\n');
  error(['The file "ngli', num2str(NL.id,'%03u'), '.rn" does not exist.']); 
end





function write_info(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.txt'], 'w');

if NL.id <= 300
  fprintf(fid, 'Ng and Li standard test-problem number %d\n\n', NL.id);
else
  fprintf(fid, 'Ng and Li user-defined test-problem number %d\n\n', NL.id);
  fprintf(fid, ['Random numbers are generated and stored in "ngli',...
               num2str(NL.id,'%03u'), '.rn"\n\n']);
end

fprintf(fid, 'Dimension:\n');
fprintf(fid, '% d\n\n', NL.n);

if NL.id <= 300
  fprintf(fid, 'Difficulty Level:\n');
  fprintf(fid, '% d\n\n', NL.level);
end

fprintf(fid, 'No. of difficult subproblems:\n');
fprintf(fid, '% d\n\n', NL.nDiff);

fprintf(fid, 'No. of easy subproblems:\n');
fprintf(fid, '% d\n\n', NL.n-NL.nDiff);


fprintf(fid, 'No. of global minimizer:\n');
fprintf(fid, ' 1\n\n');

fprintf(fid, 'No. of non-global minimizers:\n');
fprintf(fid, ' 2^%d - 1\n\n', NL.n);

fprintf(fid, 'No. of saddle points:\n');
fprintf(fid, ' 3^%d - 2^%d - 1\n\n', NL.n, NL.n);

fprintf(fid, 'No. of local maximizer:\n');
fprintf(fid, ' 1\n\n');


fprintf(fid, 'Value of the local maximizer:\n');
fprintf(fid, '% 19.15e\n\n', NL.fMax);


fprintf(fid, ['Lower bounds of var for calculating ',...
              'an upper bound of the obj value:\n']);
fprintf(fid, '% 19.15e\n', NL.yLB);
fprintf(fid, '\n');

fprintf(fid, ['Upper bounds of var for calculating ',...
              'an upper bound of the obj value:\n']);
fprintf(fid, '% 19.15e\n', NL.yUB);
fprintf(fid, '\n');

fprintf(fid, ['A loose upper bound of the obj value ',...
              'within the above variable-bounds:\n']);
fprintf(fid, '% 19.15e\n', NL.fUBUB);
fprintf(fid, '\n');


fprintf(fid, ['Minimum eigenvalue of the Hessian matrix ',...
              'at the global minimizer:\n']);
fprintf(fid, '% 19.15e\n\n', NL.ddgEigMin);

fprintf(fid, ['2-norm condition no. of the Hessian matrix ',...
              'at the global minimizer:\n']);
fprintf(fid, '% 19.15e\n\n', NL.ddgCond);


fprintf(fid, 'Global minimizer:\n');
fprintf(fid, '% 19.15e\n', NL.yStar);
fprintf(fid, '\n');

fprintf(fid, 'Global minimum value:\n');
fprintf(fid, '% 19.15e\n\n', NL.fStar);

fclose(fid);


fid = fopen('ngliglobal.txt', 'a');
fprintf(fid, '%03u\t %d\n', NL.id, NL.global);
fclose(fid);





function write_matlab(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.m'], 'w');

if NL.id <= 300
  fprintf(fid, ['%% Ng and Li standard test-problem number %d ',...
                '(n=%d, level=%d)\n'], NL.id, NL.n, NL.level);
else
  fprintf(fid, '%% Ng and Li user-defined test-problem number %d\n', NL.id);
end
fprintf(fid, '%%\n');
fprintf(fid, '%% This file stores the parameters of the named test-problem\n');
fprintf(fid, '%% for use with the function NGLIOBJ\n');
fprintf(fid, '%%\n');
if NL.id <= 300
  fprintf(fid, ...
    ['%% The global minimizer and its value ', ...
     'are stored in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
else
  fprintf(fid, ...
    '%% The parameters, the global minimizer and its value are stored\n');
  fprintf(fid, ...
    ['%% in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
end

fprintf(fid, ['function NL = ngli', num2str(NL.id,'%03u'), '\n\n']);

fprintf(fid, 'NL.n = %d;\n\n', NL.n);

fprintf(fid, 'NL.a = [\n');
fprintf(fid, '% 19.15e\n', NL.a);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.p = [\n');
fprintf(fid, '% 19.15e\n', NL.p);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.q = [\n');
fprintf(fid, '% 19.15e\n', NL.q);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.s = [\n');
fprintf(fid, '% 19.15e\n', NL.s);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.d = [\n');
fprintf(fid, '% 19.15e\n', NL.d);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.v = [\n');
fprintf(fid, '% 19.15e\n', NL.v);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.LB = [\n');
fprintf(fid, '% 19.15e\n', NL.yLB);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.UB = [\n');
fprintf(fid, '% 19.15e\n', NL.yUB);
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.val = % 19.15e;\n\n\n', NL.fStar);

fclose(fid);





function write_gams(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.gms'], 'w');


% Print TITLE statement to file
%
if NL.id <= 300
  fprintf(fid, ...
    ['$TITLE  Ng and Li standard test-problem number %d ',...
     '(n=%d, level=%d)\n\n\n'], NL.id, NL.n, NL.level);
  fprintf(fid, ...
    ['* The global minimizer and its value ',...
        'are stored in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
else
  fprintf(fid, ...
    '$TITLE  Ng and Li user-defined test-problem number %d\n\n\n', NL.id);
  fprintf(fid, ...
    '* The parameters, the global minimizer and its value are stored\n');
  fprintf(fid, ...
    ['* in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
end


% Print FILE to file
%
fprintf(fid, 'FILE\n');
fprintf(fid, ['res \t /ngli', '%03u', 'gms.txt/;\n'], NL.id);
fprintf(fid, 'PUT \t res;\n\n\n');


% Print SETS to file
%
fprintf(fid, 'SETS\n');
fprintf(fid, ['i \t /1*', '%d', '/;\n'], NL.n);
fprintf(fid, 'ALIAS(i,j);\n\n\n');


% Print PARAMETERS to file
%
fprintf(fid, 'PARAMETERS\n');

fprintf(fid, 'a(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.a(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.a(NL.n));

fprintf(fid, 'p(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.p(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.p(NL.n));

fprintf(fid, 'q(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.q(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.q(NL.n));

fprintf(fid, 's(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.s(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.s(NL.n));

fprintf(fid, 'd(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.d(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.d(NL.n));

fprintf(fid, 'v(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.v(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.v(NL.n));

fprintf(fid, 'H(i,j);\n\n');

fprintf(fid, 'LOOP(i, LOOP(j,\n');
fprintf(fid, 'H(i,j) = -2*v(i)*v(j) + 1 $ (ORD(i) EQ ORD(j));\n');
fprintf(fid, '); );\n\n\n');


% Print VARIABLES to file
%
fprintf(fid, 'VARIABLES\n');
fprintf(fid, 'y(i)\n');
fprintf(fid, 'x(i)\n');
fprintf(fid, 'obj;\n\n');

for I=1:NL.n
  fprintf(fid, 'y.lo(''%d'') =\t % 19.15e;\n', I, NL.yLB(I));
  fprintf(fid, 'y.up(''%d'') =\t % 19.15e;\n', I, NL.yUB(I));
end
fprintf(fid, '\n');

for I=1:NL.n
    fprintf(fid, 'x.lo(''%d'') =\t % 19.15e;\n', I, NL.xLB(I));
    fprintf(fid, 'x.up(''%d'') =\t % 19.15e;\n', I, NL.xUB(I));
end
fprintf(fid, '\n\n');


% Print EQUATIONS to file
%
fprintf(fid, 'EQUATIONS\n');
fprintf(fid, 'tmp_x(i)\n');
fprintf(fid, 'tmp_obj;\n\n');

fprintf(fid, 'tmp_x(i)..\n');
fprintf(fid, 'x(i) =e= d(i) * sum(j, H(i,j)*y(j));\n\n');
fprintf(fid, 'tmp_obj..\n');
fprintf(fid, ['obj =e= sum(i, ', ...
'a(i)*(power(x(i),4)+4*p(i)*power(x(i),3)+6*q(i)*sqr(x(i))+s(i)*x(i)));\n\n\n']);


% Print MODEL and SOLVE to file
%
fprintf(fid, 'MODEL testproblem /ALL/;\n');
fprintf(fid, 'testproblem.optfile   = 1;\n');
fprintf(fid, 'testproblem.workspace = 1024;\n');
fprintf(fid, 'SOLVE testproblem USING NLP MINIMIZING obj;\n\n\n');


% Print results to file
%
fprintf(fid, 'PUT "Minimum value:"/;\n');
fprintf(fid, 'PUT obj.l:22:15/;\n\n');
fprintf(fid, 'PUT "Minimizer:"/;\n');
fprintf(fid, 'LOOP(i, PUT y.l(i):22:15/);\n');


fclose(fid);
