% NGLISTD    Ng and Li's Standard Test-Problems Generator (version 2.0)
%
% NGLISTD generates a set of 300 standard test-problems, namely,
%   ngli001.m   -- ngli300.m   (for Matlab),
%   ngli001.gms -- ngli300.gms (for GAMS),
%   ngli001.txt -- ngli300.txt (problem characteristics).
% This standard set includes 10 different sizes of problems:
%   n = 2, 5, 10, 20, 50, 100, 200, 500, 1000 and 2000.
% For each size of the problems, it includes 3 levels of
% difficulties:  level = 0, 1, and 2;
% and each level consists of 10 instants of test-problems.
%
% NGLISTD generates all 300 standard test-problems.
%
% NGLISTD(ALL_ID) generates the standard test-problems with IDs listed
% in the array ALL_ID.  For example, nglistd(61:90) generates the 
% 30 standard test-problems with n=10.
%
% The following is a list of the 300 standard test-problems:
% NAMES 		   n	level
% ngli001 -- ngli010	   2	  0
% ngli011 -- ngli020	   2	  1
% ngli021 -- ngli030	   2	  2
% ngli031 -- ngli040	   5	  0
% ngli041 -- ngli050	   5	  1
% ngli051 -- ngli060	   5	  2
% ngli061 -- ngli070	  10	  0
% ngli071 -- ngli080	  10	  1
% ngli081 -- ngli090	  10	  2
% ngli091 -- ngli100	  20	  0
% ngli101 -- ngli110	  20	  1
% ngli111 -- ngli120	  20	  2
% ngli121 -- ngli130	  50	  0
% ngli131 -- ngli140	  50	  1
% ngli141 -- ngli150	  50	  2
% ngli151 -- ngli160	 100	  0
% ngli161 -- ngli170	 100	  1
% ngli171 -- ngli180	 100	  2
% ngli181 -- ngli190	 200	  0
% ngli191 -- ngli200	 200	  1
% ngli201 -- ngli210	 200	  2
% ngli211 -- ngli220	 500	  0
% ngli221 -- ngli230	 500	  1
% ngli231 -- ngli240	 500	  2
% ngli241 -- ngli250	1000	  0
% ngli251 -- ngli260	1000	  1
% ngli261 -- ngli270	1000	  2
% ngli271 -- ngli280	2000	  0
% ngli281 -- ngli290	2000	  1
% ngli291 -- ngli300	2000	  2



function nglistd(AllID)



StartTime = cputime;



if nargin == 0
  AllID = 1:300;
elseif isempty(AllID) || any(AllID > 300)
  error('Invalid standard test-problem IDs.');
end



NL.aLB =  1;
NL.aUB =  2;

NL.pUB =  1;
NL.pLB = -NL.pUB;

NL.qLB = -2;
NL.qUB = -1;

NL.alphaPrime = 0.95;

NL.dLB = 0.25;
NL.dUB = 0.50;

NL.deltaLB = 0.3;
NL.deltaUB = 0.7;




for ID = AllID
  NL.id    = ID;
  NL.class = floor((ID-1)/30);

  switch NL.class
  case 0
    NL.n = 2;
  case 1
    NL.n = 5;
  case 2
    NL.n = 10;
  case 3
    NL.n = 20;
  case 4
    NL.n = 50;
  case 5
    NL.n = 100;
  case 6
    NL.n = 200;
  case 7
    NL.n = 500;
  case 8
    NL.n = 1000;
  case 9
    NL.n = 2000;
  end

  NL.level = floor((ID-1-NL.class*30)/10);	% level = 0:2
  NL.nDiff = round(NL.n * NL.level/2);

  fprintf(['Generating test-problem number %03d ', ...
           '(n=%04d, instant=%02d)... '], ...
          NL.id, NL.n, mod(NL.id-1,30)+1);
  NL = nglimain(NL);
  fprintf('Done.\n');
end



EndTime = cputime;
fprintf('Time Elapsed = %f\n', EndTime-StartTime);
